#
# Copyright © 2012 - 2019 Michal Čihař <michal@cihar.com>
#
# This file is part of Weblate <https://weblate.org/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

from django.test.utils import modify_settings
from weblate.auth.models import User
from weblate.trans.models import Change, Project
from weblate.trans.tests.test_views import FixtureTestCase

from .tasks import get_change_body, get_change_topic


class FedoraTestCase(FixtureTestCase):
    def test_topic(self):
        for change in Change.objects.all():
            self.assertIsNotNone(get_change_topic(change))

    def test_body(self):
        for change in Change.objects.all():
            self.assertIsNotNone(get_change_body(change))

    @modify_settings(INSTALLED_APPS={"append": "weblate_fedora_messaging"})
    def test_create(self):
        user = User.objects.get(username="testuser")
        project = Project.objects.all()[0]
        Change.objects.create(
            action=Change.ACTION_REMOVE_PROJECT, target="test", user=user
        )
        Change.objects.create(
            action=Change.ACTION_REMOVE_COMPONENT,
            project=project,
            target="test",
            user=user,
            author=User.objects.get(username="jane"),
        )
        self.edit_unit("Hello, world!\n", "Ahoj svete!\n")



#
# Copyright © 2012 - 2019 Michal Čihař <michal@cihar.com>
#
# This file is part of Weblate <https://weblate.org/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
from __future__ import unicode_literals

import os

import fedora_messaging
from django.apps import AppConfig
from django.conf import settings


def configure_fedora_messaging():
    if settings.FEDORA_MESSAGING_CONF:
        fedora_messaging.config.conf.load_config(
            config_path=settings.FEDORA_MESSAGING_CONF
        )
    if "CI_AMQP_HOST" in os.environ:
        fedora_messaging.config.conf.update(
            amqp_url="amqp://{}?connection_attempts=3&retry_delay=5".format(
                os.environ["CI_AMQP_HOST"]
            )
        )
    fedora_messaging.config.conf.setup_logging()


class FedoraConfig(AppConfig):
    name = "weblate_fedora_messaging"
    label = "weblate_fedora_messaging"
    verbose_name = "Weblate Fedora Messaging"

    def ready(self):
        super().ready()
        configure_fedora_messaging()



#
# Copyright © 2012 - 2019 Michal Čihař <michal@cihar.com>
#
# This file is part of Weblate <https://weblate.org/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

from django.core.exceptions import ObjectDoesNotExist
from fedora_messaging.api import Message, publish
from fedora_messaging.exceptions import ConnectionException, PublishReturned
from weblate.trans.models import Change
from weblate.utils.celery import app
from weblate.utils.site import get_site_url


def get_change_topic(change):
    scope = "global"
    if change.unit:
        scope = "unit"
    elif change.translation:
        scope = "translation"
    elif change.component:
        scope = "component"
    elif change.project:
        scope = "project"
    return "weblate.{}.{}".format(
        scope, change.get_action_display().lower().replace(" ", "_")
    )


def get_change_body(change):
    result = {
        "id": change.id,
        "action": change.get_action_display(),
        "timestamp": change.timestamp.isoformat(),
    }
    url = change.get_absolute_url()
    if url:
        result["url"] = get_site_url(url)
    if change.target:
        result["target"] = change.target
    if change.old:
        result["old"] = change.old
    if change.author:
        result["author"] = change.author.username
    if change.user:
        result["user"] = change.user.username
    if change.project:
        result["project"] = change.project.slug
    if change.component:
        result["component"] = change.component.slug
    if change.translation:
        result["translation"] = change.translation.language.code
    if change.unit:
        result["source"] = change.unit.source
    result.update(change.details)
    return result


# Retry for not existing object (maybe transaction not yet committed) with
# delay of 10 minutes growing exponentially
@app.task(
    trail=False,
    autoretry_for=(ObjectDoesNotExist, PublishReturned, ConnectionException),
    retry_backoff=600,
    retry_backoff_max=3600,
)
def fedora_messaging_change(change_id):
    change = Change.objects.get(pk=change_id)
    publish(Message(topic=get_change_topic(change), body=get_change_body(change)))



#
# Copyright © 2012 - 2019 Michal Čihař <michal@cihar.com>
#
# This file is part of Weblate <https://weblate.org/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#

from appconf import AppConf
from django.db.models.signals import post_save
from django.dispatch import receiver
from weblate.trans.models import Change
from weblate.utils.decorators import disable_for_loaddata

from .tasks import fedora_messaging_change


@receiver(post_save, sender=Change)
@disable_for_loaddata
def fedora_notify_change(sender, instance, **kwargs):
    fedora_messaging_change.delay(instance.pk)


class FedoraConf(AppConf):

    FEDORA_MESSAGING_CONF = None

    class Meta:
        prefix = ""



#!/usr/bin/env python
#
# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
#
# This file is part of Weblate <https://weblate.org/>
#

from setuptools import setup

with open("README.rst") as handle:
    LONG_DESCRIPTION = handle.read()

with open("requirements.txt") as handle:
    REQUIRES = handle.read().split()

with open("requirements-test.txt") as handle:
    REQUIRES_TEST = handle.read().split()

setup(
    name="weblate_fedora_messaging",
    version="0.2",
    author="Michal Čihař",
    author_email="michal@cihar.com",
    description="Weblate Fedora Messaging integration",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/x-rst",
    license="GPLv3+",
    keywords="i18n l10n gettext git mercurial translate",
    url="https://weblate.org/",
    download_url="https://github.com/WeblateOrg/fedora_mesaging",
    project_urls={
        "Issue Tracker": "https://github.com/WeblateOrg/fedora_messaging/issues",
        "Documentation": "https://docs.weblate.org/",
        "Source Code": "https://github.com/WeblateOrg/fedora_messaging",
        "Twitter": "https://twitter.com/WeblateOrg",
    },
    platforms=["any"],
    packages=["weblate_fedora_messaging"],
    include_package_data=True,
    install_requires=REQUIRES,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Internationalization",
        "Topic :: Software Development :: Localization",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    python_requires=">=3.5",
    setup_requires=["pytest-runner"],
    tests_require=REQUIRES_TEST,
)
